/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 */

#include "edir.h"

#ifdef EDIR_CACHE
/*
 * edir cache table
 */
edir_cache_table_t edir_cache;

/*
 * cache enable flag
 */
int32 edir_cache_enabled = 0;

int32 edir_cache_init(edir_cache_table_t *edir_cache,
					  int32 cachesize,
					  char *pattern,
					  char *flush)
{
	EDIR_DEBUG_CACHE(("EnhydraDirector Cache initalization"));
	EDIR_DEBUG_CACHE_VERBOSE(("number of cache entries:%d",(int)EDIR_CACHE_ENTRIES));
	memset(edir_cache, 0, sizeof(edir_cache_table_t));
	edir_cache->num_entries = EDIR_CACHE_ENTRIES;
	edir_cache->num_free_entries = EDIR_CACHE_ENTRIES;
	edir_cache->max_size = cachesize;

	if (strlen(pattern)>=512) 
	{
 		strncpy(edir_cache->pattern, pattern, 512);
		edir_cache->pattern[512-1] = '\0';
	}
	else
		strcpy(edir_cache->pattern, pattern);
	strcpy(edir_cache->cacheflush, flush);
	edir_cache->cachectrl = NULL;
	return 1;
}

void edir_cache_status(edir_cache_table_t *edir_cache, void *sbuf, edir_exception_t **exc)
{
	int32 idx;

	if (edir_cache_enabled)
	{
		edir_sbuf_appendf(sbuf, "\nEnhydra Cache: enabled\n");
		edir_sbuf_appendf(sbuf, "  number of table entries: %d\n", edir_cache->num_entries);
		edir_sbuf_appendf(sbuf, "  number of free entries: %d\n", edir_cache->num_free_entries);
		edir_sbuf_appendf(sbuf, "  max cache size: %dKB\n", edir_cache->max_size);
		edir_sbuf_appendf(sbuf, "  current cache size: %dKB\n", edir_cache->size);
		edir_sbuf_appendf(sbuf, "  pattern list: %s\n\n", edir_cache->pattern);

		for (idx=0; idx<EDIR_CACHE_ENTRIES; idx++)
		{
			if (edir_cache->entry[idx].status == EDIR_CACHE_ENTRY_STATUS_VALID)
				edir_sbuf_appendf(sbuf, "  entry #%d: uri=%s\n    age=%d\n    time=%d\n", \
					idx,
					edir_cache->entry[idx].uri,
					edir_cache->entry[idx].age,
					time(NULL) - edir_cache->entry[idx].ctime);
			else if (edir_cache->entry[idx].status == EDIR_CACHE_ENTRY_STATUS_RESERVED)
				edir_sbuf_appendf(sbuf, "  entry #%d: reserved for uri=%s\n    age=%d\n", \
					idx, edir_cache->entry[idx].uri, edir_cache->entry[idx].age);
			else
				edir_sbuf_appendf(sbuf, "  entry #%d: empty\n", idx);
		}
	}
	else
	{
		edir_sbuf_appendf(sbuf, "Enhydra Cache: disabled\n");
	}
}

int32 edir_cache_flush(edir_cache_table_t *edir_cache)
{
	int32 i;

	EDIR_DEBUG_CACHE_VERBOSE(("flushing cache entries."));
	for (i=0; i<EDIR_CACHE_ENTRIES; i++) {
		if (edir_cache->entry[i].status == EDIR_CACHE_ENTRY_STATUS_VALID)
		{
			edir_cache->entry[i].status = EDIR_CACHE_ENTRY_STATUS_EMPTY;
		}
	}

	return 0;
}

int32 edir_cache_lookup(edir_cache_table_t *edir_cache,
						char *request_uri, int32 *cache_idx)
{
	int32 i;
	int32 find = 0;
	int32 max_age, max_find;

	i = 0;

	EDIR_DEBUG_CACHE_VERBOSE(("finding a match..."));
	while (!find && i<EDIR_CACHE_ENTRIES)
	{
		if (edir_cache->entry[i].status == EDIR_CACHE_ENTRY_STATUS_VALID)
		{
			if (strcmp(edir_cache->entry[i].uri, request_uri) == 0)
			{
				EDIR_DEBUG_CACHE_VERBOSE(("match found."));
				find = 1;
				*cache_idx = i;
				edir_cache->entry[i].age = -1;
			}
		}
		i++;
	}
	
	EDIR_DEBUG_CACHE_VERBOSE(("updating entries age"));
	for (i=0; i<EDIR_CACHE_ENTRIES; i++) {
		edir_cache->entry[i].age += 1;
	}

	if (!find)
	{
		if (edir_cache->num_free_entries > 0)
		{
			EDIR_DEBUG_CACHE_VERBOSE(("find an empty entry..."));
			for (i=0; i<EDIR_CACHE_ENTRIES; i++)
			{
				if (edir_cache->entry[i].status == EDIR_CACHE_ENTRY_STATUS_EMPTY)
				{
					EDIR_DEBUG_CACHE_VERBOSE(("empty entry found."));
					*cache_idx = i;
					break;
				}
			}
		}
		else
		{
			EDIR_DEBUG_CACHE_VERBOSE(("find an old valid entry to empty..."));
			max_age = 0;
			max_find = 0;
			for (i=0; i<EDIR_CACHE_ENTRIES; i++)
			{
				if ((edir_cache->entry[i].status == EDIR_CACHE_ENTRY_STATUS_VALID) &&
					(edir_cache->entry[i].age > max_age))
				{
					max_age = edir_cache->entry[i].age;
					*cache_idx = i;
					max_find = 1;
				}
			}
			if (!max_find)
			{
				EDIR_DEBUG_CACHE_VERBOSE(("not found. all reserverd!!!"));
				*cache_idx = (int32)EDIR_CACHE_ENTRIES;
			}
			else
				EDIR_DEBUG_CACHE_VERBOSE(("found an old valid entry to empty."));
		}
	}
	EDIR_DEBUG_CACHE_VERBOSE(("return: find=%d, idx=%d", find, *cache_idx));
	return find;
}

int32 edir_cache_pattern_match(edir_cache_table_t *edir_cache,
							   char *request_uri)
{
	char *pl, *pc, *pt, *istr;
	int32 ilen;
	char pattern[EDIR_CACHE_PATTERN_STORAGE_SIZE];
	
	strcpy(pattern, edir_cache->pattern);
	
	istr = edir_strtrim(request_uri);
	ilen = strlen(request_uri);
	pl = pattern;
	pc = pattern;

	EDIR_DEBUG_CACHE_VERBOSE(("comparing %s with:", request_uri));

	while (*pc)
	{
		while (*pl && *pl!=' ') pl++;
		if (*pl)
		{
			*(pl++) = '\0';
			if (ilen > (int32)strlen(pc))
			{
				pt = istr + (ilen - strlen(pc));
				EDIR_DEBUG_CACHE_VERBOSE(("  %s (suffix %s)", pc, pt));

				if (edir_strcasecmp(pt, pc)==0)
				{
					edir_free(istr);
					return 1;
				}
			}
			pc = pl;
		}
		else
		{
			if (pc == pl) break;
			if (ilen > (int32)strlen(pc))
			{
				pt = istr + (ilen - strlen(pc));
				EDIR_DEBUG_CACHE_VERBOSE(("  %s (suffix %s)", pc, pt));

				if (edir_strcasecmp(pt, pc)==0)
				{
					edir_free(istr);
					return 1;
				}
			}
			pc = pl;
		}
	}
	if (istr) edir_free(istr);

	return 0;
}

int32 edir_cache_check_flushpath(char *flushpath,
							   char *request_uri)
{
	char *uri, *s;
	int32 ret;

	uri = edir_strdup(request_uri);
	s = uri;

	while (s && *s!='\0')
		if (*s=='/' && *(s+1)=='\0')
			*s = '\0';
		else
			s++;
	ret = edir_strcasecmp(flushpath, uri+1);
	edir_free(uri);
	return (ret) ? 0 : 1;
}

#endif