/*
 * Enhydra Java Application Server 
 * The Initial Developer of the Original Code is Lutris Technologies Inc. 
 * Portions created by Lutris are Copyright (C) 1997-2000 Lutris Technologies
 * Inc. 
 * All Rights Reserved. 
 *
 * The contents of this file are subject to the Enhydra Public License Version
 * 1.0 (the "License"); you may not use this file except in compliance with the
 * License. You may obtain a copy of the License at
 * http://www.enhydra.org/software/license/epl.html 
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under the
 * License. 
 *
 */

#ifndef __EDIR_CACHE_H__
#define __EDIR_CACHE_H__ 1
#ifdef __cplusplus
extern "C" {
#endif

#define EDIR_CACHE_ENTRIES					(20)
#define EDIR_CACHE_URI_MAX_LENGTH			(1024)
#define EDIR_CACHE_MAX_SIZE					(512)
#define EDIR_CACHE_PATTERN_STORAGE_SIZE		(128)

#define EDIR_CACHE_ENTRY_STATUS_EMPTY		(0)
#define EDIR_CACHE_ENTRY_STATUS_VALID		(1)
#define EDIR_CACHE_ENTRY_STATUS_RESERVED	(2)

/*
 * edir_cache_entry describes one cache entry in edir_cache_table
 */
typedef struct edir_cache_entry {
/*	int32	id;						/* cache table entry id number */
/*	struct edir_cache_entry_t *next;/* for entry enqueuing */
/*	struct edir_cache_entry_t *prev;/* for entry enqueuing */
	int32	status;					/* entry's status - empty, valid */
	int32	age;					/* entry's age */
	int32	ctime;					/* entry's creation time */
	int32	uri_len;				/* size of uri string */
	char	*uri;					/* entry's uri string */
	int32	statusline_len;			/* status line buffer length */
	char	*statusline;			/* status line buffer */
	int32	headers_len;			/* headers length */
	char	*headers;				/* headers buffer */
	int32	data_len;				/* data length */
	int32	data_chunk_size;
	char	*data;					/* entry's data */
	char    buf[64000];
/*	char	uristr[EDIR_CACHE_URI_MAX_LENGTH]; /* entry's uri */
} edir_cache_entry_t;

typedef struct edir_cache_table {
	int32 max_size;					/* max size of cache in KB */
	int32 size;						/* current size of cache in KB */
	int32 num_entries;				/* the number of entries */
	int32 num_free_entries;			/* the number of free entries */
	edir_conf_cache_t **cachectrl;
	char pattern[512];					/* space separated list of suffix patterns */
	char cacheflush[512];
	edir_cache_entry_t entry[EDIR_CACHE_ENTRIES];
} edir_cache_table_t;

/*
 * edir cache table
 */
extern edir_cache_table_t edir_cache;

/*
 * cache enable flag
 */
extern int32 edir_cache_enabled;

extern int32 edir_cache_init(edir_cache_table_t *edir_cache,
					  int32 cachesize,
					  char *pattern,
					  char *flush);

void edir_cache_status(edir_cache_table_t *edir_cache,
					   void *sbuf, edir_exception_t **exc);

int32 edir_cache_lookup(edir_cache_table_t *edir_cache,
						char *request_uri, int32 *cache_idx);

int32 edir_cache_pattern_match(edir_cache_table_t *edir_cache,
							   char *request_uri);

int32 edir_cache_check_flushpath(char *flushpath,
							   char *request_uri);

int32 edir_cache_flush(edir_cache_table_t *edir_cache);

#ifdef __cplusplus
} /* Extern "C" */
#endif
#endif /* __EDIR_CACHE_H__ */
